@@ -1,12 +1,37 @@
-from dataclasses import asdict, dataclass
+import mimetypes
+from dataclasses import asdict, dataclass, field, fields
 from datetime import datetime
 from enum import IntEnum
-from typing import Any, Callable, Dict, Generic, List, Optional, TypeVar
+from os import PathLike
+from pathlib import Path
+from typing import IO, Any, Callable, ClassVar, Dict, Generic, List, Literal, Optional, TypeVar, Union
+from typing_extensions import TypeAlias
 
 from .element import Element, transform
+from .parser import Element as RawElement
 from .parser import parse
 
 
+@dataclass
+class ModelBase:
+    __converter__: ClassVar[Dict[str, Callable[[Any], Any]]] = {}
+
+    @classmethod
+    def parse(cls, raw: dict):
+        fs = fields(cls)
+        data = {}
+        for fd in fs:
+            if fd.name in raw:
+                if fd.name in cls.__converter__:
+                    data[fd.name] = cls.__converter__[fd.name](raw[fd.name])
+                else:
+                    data[fd.name] = raw[fd.name]
+        return cls(**data)  # type: ignore
+
+    def dump(self) -> dict:
+        raise NotImplementedError
+
+
 class ChannelType(IntEnum):
     TEXT = 0
     DIRECT = 1
@@ -15,17 +40,13 @@ class ChannelType(IntEnum):
 
 
 @dataclass
-class Channel:
+class Channel(ModelBase):
     id: str
     type: ChannelType
     name: Optional[str] = None
     parent_id: Optional[str] = None
 
-    @classmethod
-    def parse(cls, raw: dict):
-        data = raw.copy()
-        data["type"] = ChannelType(raw["type"])
-        return cls(**data)
+    __converter__ = {"type": ChannelType}
 
     def dump(self):
         res = {"id": self.id, "type": self.type.value}
@@ -37,15 +58,11 @@ class Channel:
 
 
 @dataclass
-class Guild:
+class Guild(ModelBase):
     id: str
     name: Optional[str] = None
     avatar: Optional[str] = None
 
-    @classmethod
-    def parse(cls, raw: dict):
-        return cls(**raw)
-
     def dump(self):
         res = {"id": self.id}
         if self.name:
@@ -56,17 +73,13 @@ class Guild:
 
 
 @dataclass
-class User:
+class User(ModelBase):
     id: str
     name: Optional[str] = None
     nick: Optional[str] = None
     avatar: Optional[str] = None
     is_bot: Optional[bool] = None
 
-    @classmethod
-    def parse(cls, raw: dict):
-        return cls(**raw)
-
     def dump(self):
         res: Dict[str, Any] = {"id": self.id}
         if self.name:
@@ -81,28 +94,20 @@ class User:
 
 
 @dataclass
-class Member:
+class Member(ModelBase):
     user: Optional[User] = None
     nick: Optional[str] = None
-    name: Optional[str] = None
     avatar: Optional[str] = None
     joined_at: Optional[datetime] = None
 
-    @classmethod
-    def parse(cls, raw: dict):
-        data = raw.copy()
-        if "user" in raw:
-            data["user"] = User.parse(raw["user"])
-        if "joined_at" in raw:
-            data["joined_at"] = datetime.fromtimestamp(int(raw["joined_at"]) / 1000)
-        return cls(**data)
+    __converter__ = {"user": User.parse, "joined_at": lambda ts: datetime.fromtimestamp(int(ts) / 1000)}
 
     def dump(self):
         res = {}
         if self.user:
             res["user"] = self.user.dump()
-        if self.nick or self.name:
-            res["nick"] = self.nick or self.name
+        if self.nick:
+            res["nick"] = self.nick
         if self.avatar:
             res["avatar"] = self.avatar
         if self.joined_at:
@@ -111,14 +116,10 @@ class Member:
 
 
 @dataclass
-class Role:
+class Role(ModelBase):
     id: str
     name: Optional[str] = None
 
-    @classmethod
-    def parse(cls, raw: dict):
-        return cls(**raw)
-
     def dump(self):
         res = {"id": self.id}
         if self.name:
@@ -135,22 +136,22 @@ class LoginStatus(IntEnum):
 
 
 @dataclass
-class Login:
+class Login(ModelBase):
     status: LoginStatus
     user: Optional[User] = None
     self_id: Optional[str] = None
     platform: Optional[str] = None
+    features: List[str] = field(default_factory=list)
+    proxy_urls: List[str] = field(default_factory=list)
 
-    @classmethod
-    def parse(cls, raw: dict):
-        data = raw.copy()
-        if "user" in raw:
-            data["user"] = User(**raw["user"])
-        data["status"] = LoginStatus(data["status"])
-        return cls(**data)
+    __converter__ = {"user": User.parse, "status": LoginStatus}
 
     def dump(self):
-        res: Dict[str, Any] = {"status": self.status.value}
+        res: Dict[str, Any] = {
+            "status": self.status.value,
+            "features": self.features,
+            "proxy_urls": self.proxy_urls,
+        }
         if self.user:
             res["user"] = self.user.dump()
         if self.self_id:
@@ -161,7 +162,7 @@ class Login:
 
 
 @dataclass
-class ArgvInteraction:
+class ArgvInteraction(ModelBase):
     name: str
     arguments: list
     options: Any
@@ -171,7 +172,7 @@ class ArgvInteraction:
 
 
 @dataclass
-class ButtonInteraction:
+class ButtonInteraction(ModelBase):
     id: str
 
     def dump(self):
@@ -187,18 +188,18 @@ class Opcode(IntEnum):
 
 
 @dataclass
-class Identify:
+class Identify(ModelBase):
     token: Optional[str] = None
     sequence: Optional[int] = None
 
 
 @dataclass
-class Ready:
+class Ready(ModelBase):
     logins: List[Login]
 
 
 @dataclass
-class MessageObject:
+class MessageObject(ModelBase):
     id: str
     content: str
     channel: Optional[Channel] = None
@@ -208,29 +209,39 @@ class MessageObject:
     created_at: Optional[datetime] = None
     updated_at: Optional[datetime] = None
 
+    @classmethod
+    def from_elements(
+        cls,
+        id: str,
+        content: List[Element],
+        channel: Optional[Channel] = None,
+        guild: Optional[Guild] = None,
+        member: Optional[Member] = None,
+        user: Optional[User] = None,
+        created_at: Optional[datetime] = None,
+        updated_at: Optional[datetime] = None,
+    ):
+        return cls(id, "".join(str(i) for i in content), channel, guild, member, user, created_at, updated_at)
+
     @property
     def message(self) -> List[Element]:
         return transform(parse(self.content))
 
     @classmethod
     def parse(cls, raw: dict):
-        data = {
-            "id": raw["id"],
-            "content": raw["content"],
-        }
-        if "channel" in raw:
-            data["channel"] = Channel.parse(raw["channel"])
-        if "guild" in raw:
-            data["guild"] = Guild.parse(raw["guild"])
-        if "member" in raw:
-            data["member"] = Member.parse(raw["member"])
-        if "user" in raw:
-            data["user"] = User.parse(raw["user"])
-        if "created_at" in raw:
-            data["created_at"] = datetime.fromtimestamp(int(raw["created_at"]) / 1000)
-        if "updated_at" in raw:
-            data["updated_at"] = datetime.fromtimestamp(int(raw["updated_at"]) / 1000)
-        return cls(**data)
+        if "elements" in raw and "content" not in raw:
+            content = [RawElement(*item.values()) for item in raw["elements"]]
+            raw["content"] = "".join(str(i) for i in content)
+        return super().parse(raw)
+
+    __converter__ = {
+        "channel": Channel.parse,
+        "guild": Guild.parse,
+        "member": Member.parse,
+        "user": User.parse,
+        "created_at": lambda ts: datetime.fromtimestamp(int(ts) / 1000),
+        "updated_at": lambda ts: datetime.fromtimestamp(int(ts) / 1000),
+    }
 
     def dump(self):
         res: Dict[str, Any] = {"id": self.id, "content": self.content}
@@ -250,7 +261,7 @@ class MessageObject:
 
 
 @dataclass
-class Event:
+class Event(ModelBase):
     id: int
     type: str
     platform: str
@@ -267,36 +278,22 @@ class Event:
     role: Optional[Role] = None
     user: Optional[User] = None
 
-    @classmethod
-    def parse(cls, raw: dict):
-        data = {
-            "id": raw["id"],
-            "type": raw["type"],
-            "platform": raw["platform"],
-            "self_id": raw["self_id"],
-            "timestamp": datetime.fromtimestamp(int(raw["timestamp"]) / 1000),
-        }
-        if "argv" in raw:
-            data["argv"] = ArgvInteraction(**raw["argv"])
-        if "button" in raw:
-            data["button"] = ButtonInteraction(**raw["button"])
-        if "channel" in raw:
-            data["channel"] = Channel.parse(raw["channel"])
-        if "guild" in raw:
-            data["guild"] = Guild.parse(raw["guild"])
-        if "login" in raw:
-            data["login"] = Login.parse(raw["login"])
-        if "member" in raw:
-            data["member"] = Member.parse(raw["member"])
-        if "message" in raw:
-            data["message"] = MessageObject.parse(raw["message"])
-        if "operator" in raw:
-            data["operator"] = User.parse(raw["operator"])
-        if "role" in raw:
-            data["role"] = Role.parse(raw["role"])
-        if "user" in raw:
-            data["user"] = User.parse(raw["user"])
-        return cls(**data)
+    _type: Optional[str] = None
+    _data: Optional[dict] = None
+
+    __converter__ = {
+        "timestamp": lambda ts: datetime.fromtimestamp(int(ts) / 1000),
+        "argv": ArgvInteraction.parse,
+        "button": ButtonInteraction.parse,
+        "channel": Channel.parse,
+        "guild": Guild.parse,
+        "login": Login.parse,
+        "member": Member.parse,
+        "message": MessageObject.parse,
+        "operator": User.parse,
+        "role": Role.parse,
+        "user": User.parse,
+    }
 
     def dump(self):
         res = {
@@ -326,18 +323,70 @@ class Event:
             res["role"] = self.role.dump()
         if self.user:
             res["user"] = self.user.dump()
+        if self._type:
+            res["_type"] = self._type
+        if self._data:
+            res["_data"] = self._data
         return res
 
 
-T = TypeVar("T")
+T = TypeVar("T", bound=ModelBase)
 
 
 @dataclass
-class PageResult(Generic[T]):
+class PageResult(ModelBase, Generic[T]):
     data: List[T]
     next: Optional[str] = None
 
     @classmethod
-    def parse(cls, raw: dict, parser: Callable[[dict], T]) -> "PageResult[T]":
-        data = [parser(item) for item in raw["data"]]
+    def parse(cls, raw: dict, parser: Optional[Callable[[dict], T]] = None) -> "PageResult[T]":
+        data = [(parser or ModelBase.parse)(item) for item in raw["data"]]
         return cls(data, raw.get("next"))
+
+    def dump(self):
+        res: dict = {"data": [item.dump() for item in self.data]}
+        if self.next:
+            res["next"] = self.next
+        return res
+
+
+@dataclass
+class PageDequeResult(PageResult[T]):
+    prev: Optional[str] = None
+
+    @classmethod
+    def parse(cls, raw: dict, parser: Optional[Callable[[dict], T]] = None) -> "PageDequeResult[T]":
+        data = [(parser or ModelBase.parse)(item) for item in raw["data"]]
+        return cls(data, raw.get("next"), raw.get("prev"))
+
+    def dump(self):
+        res: dict = {"data": [item.dump() for item in self.data]}
+        if self.next:
+            res["next"] = self.next
+        if self.prev:
+            res["prev"] = self.prev
+        return res
+
+
+Direction: TypeAlias = Literal["before", "after", "around"]
+Order: TypeAlias = Literal["asc", "desc"]
+
+
+@dataclass
+class Upload:
+    file: Union[bytes, IO[bytes], PathLike]
+    mimetype: str = "image/png"
+    name: Optional[str] = None
+
+    def __post_init__(self):
+        if isinstance(self.file, PathLike):
+            self.mimetype = mimetypes.guess_type(str(self.file))[0] or self.mimetype
+            self.name = Path(self.file).name
+
+    def dump(self):
+        file = self.file
+
+        if isinstance(file, PathLike):
+            file = open(file, "rb")
+
+        return {"value": file, "filename": self.name, "content_type": self.mimetype}
